var canFilter = true;

(function($) {

    var defaults = {
        step: 5,
        speed: 800
    };

    var GONE = 1;
    var SHIFT = 2;
    var CROSS = 3;
    var POP = 4;

    var pushLeft = '-=1200px';
    var pushRight = '+=1200px';
    var $next, $prev, $prevol, $nextol, $paginationCount, $children, $container;
    var upper, lower, page, step, count, pageLoad, speed, previousLowerBound, previousUpperBound, changedUpperBound, filteredItemsToDeleteLength;
    var right;
    var left;
    var showNext;
    var showPrev;
    var currentNumberBeingShown = 0;

    var currentlyVisibleContent = [];
    var itemsToBringInAnimation = [];
    var itemsToDelete = [];
    var filteredItemsToDelete = [];


    var methods = {
        init : function(options) {
            options = $.extend(defaults, options);
            speed = options.speed;
            pageLoad = true;
            step = options.step;

            $container = this;
            $children = this.children();
            count = $children.length;
            page = 1;

            // insert pagination link wrappers
            $prevol = $('<ol id="pagination"></ol>').insertBefore($container);
            $nextol = $('<ol id="pagination"></ol>').insertAfter($container);
            $paginationCount = $('div#pagination-count');

            right = left = showNext = showPrev = changedUpperBound = false;

            if (count > step) {
                appendNext();
            }

            $('h3#filter').bind('click', function () {
                if (canFilter) {
                    canFilter = false;

                    var newTiles = [];
                    var tags = _collectTags();

                    var filteredTiles = [];
                    if (tags == 'sport') {
                        filteredTiles = [2];
                    } else if (tags == 'sport,television') {
                        filteredTiles = [10, 13];
                    }

                    if (filteredTiles.length > 0 || newTiles.length > 0) {
                        $.fn.paginate('animateTiles', $('#tiles ul li'), filteredTiles, newTiles);
                    }
                }
            });

            show();
            // end init
        },

        animateTiles : function($tiles, filteredTiles, newContent) {

            var animationQueueNewItems = [];
            var animationQueue = []; 		// store animation parameters for better performance
            var availablePositions = [];    // available positions on screen
            var availablePositionIndex = 0; // index of the next available position to fill
            var filteredTilesIndex = 0; 	// index for the filtered items
            var numberOfReplacementsFound = 0;
            var removedItems = [];        // what is removed from the right

            var emptyLocationsToFill = 0; // what is removed from the current screen

            // get position of visible tiles between upper & lower
            for (var idx = lower; idx < upper; idx++) {
                availablePositions.push($($tiles[idx]).position());
            }

            currentlyVisibleContent = [];

            // iterate through the tiles
            $tiles.each(function (index) {
                var id = $(this).attr('data-id');
                if (index >= lower && index < upper) {
                    if (id == filteredTiles[filteredTilesIndex]) {
                        animationQueue.push({ element: $(this), type: GONE });
                        filteredItemsToDelete.push(id);
                        filteredTilesIndex++;
                        emptyLocationsToFill++;
                    }
                    else {
                        // adjust position of remaining items
                        var nextEmptySlot = availablePositions[availablePositionIndex];
                        var currPos = $(this).position();
                        var xOffset = currPos['top'] - nextEmptySlot['top'];
                        var yOffset = currPos['left'] - nextEmptySlot['left'];

                        // if something is moving right add shrinking effect
                        // x-coordinate (xOffset) will not be zero since height is changing
                        if (xOffset > 0) {
                            animationQueue.push({ element: $(this), type: CROSS, y: yOffset, x: xOffset });
                        }
                        else {
                            animationQueue.push({ element: $(this), animation: { top: "-=" + xOffset, left: "-=" + yOffset }, type: SHIFT });
                        }
                        currentlyVisibleContent.push($(this));
                        availablePositionIndex++;
                    }
                }
                else if (index < lower) {
                    // item removed from the left of current screen
                    if (id == filteredTiles[filteredTilesIndex]) {
                        itemsToDelete.push(id);
                        filteredTilesIndex++;
                    }
                }
                else {
                    if (id == filteredTiles[filteredTilesIndex]) {
                        // items to the right
                        animationQueue.push({ element: $(this), type: POP });
                        filteredTilesIndex++;
                        removedItems.push(id); // keep track of the index of an item that cannot be used as a "move-over" item
                        filteredItemsToDelete.push(id);
                    }
                }
            });

            // add any new items
            if (newContent != null && newContent.length > 0) {
                var tiles = "";
                $.each(newContent, function (key, value) {
                    tiles += createTileHTML(value, true);
                    animationQueueNewItems.push({ id: value.ContentItemID, type: 'add' });
                });

                // append only once as this is expensive
                $(tiles).appendTo($container);
            }

            filteredItemsToDeleteLength = filteredItemsToDelete.length;

            // fill in empty spaces
            if (emptyLocationsToFill > 0) {

                var counter = 0;
                var numberOfItemsToRemove = removedItems.length;
                var totalTiles = $tiles.length;

                // start searching from the next screen to the right
                for (var ind = upper; ind < totalTiles; ind++) {

                    // stop when all slots are filled or when remaining tiles == number of off screen items to remove
                    if (emptyLocationsToFill == numberOfReplacementsFound || ((totalTiles - upper) == numberOfItemsToRemove)) {
                        break;
                    }

                    if ($($tiles[ind]).attr('data-id') != removedItems[counter]) {
                        // add these to currently visible items
                        currentlyVisibleContent.push($tiles[ind]);
                        itemsToBringInAnimation.push({ element: $($tiles[ind]) });
                        numberOfReplacementsFound++;
                    } else {
                        counter++;
                    }
                }
            }

            // position next links absolute for the duration of the transition
            if ($next != null) $next.css({ 'position': 'absolute', 'right': '10px', 'bottom': '0px' });

            var lastItemIndex = animationQueue.length - 1;

            // run
            if (animationQueue.length == 0) {
                _cleanDOM(filteredTilesIndex, animationQueueNewItems,  $tiles);
                return;
            }

            for (var i = 0; i < animationQueue.length; i++) {
                var anim = null;

                if (animationQueue[i].type === GONE) {
                    anim = gone(animationQueue[i]);
                }
                else if (animationQueue[i].type === CROSS) {
                    anim = shrink(animationQueue[i]);
                }
                else if (animationQueue[i].type == SHIFT) {
                    anim = move(animationQueue[i]);
                }
                else if (animationQueue[i].type == POP) {
                    anim = removeItem(animationQueue[i]);
                }

                if (i == lastItemIndex) {
                    setTimeout(function() {
                        _cleanDOM(filteredTilesIndex, animationQueueNewItems, $tiles);
                    }, 800);
                }
            }
        }
    };

    function createTileHTML(data, hidden) {
        var mainContent, url;

        switch (data.mediaType.toLowerCase()) {
            case 'youtube':
                mainContent = '<div class="image-wrap pink"><img src="../Content/img/youtube.png" alt="A Youtube video"></div>';
                break;
            case 'bingvideo':
                mainContent = '<div class="image-wrap green"><img src="../Content/img/bing.png" alt="A Bing video"></div>';
                url = data.URL;
                break;
            case 'pdf':
                mainContent = '<div class="image-wrap yellow"><img src="../Content/img/pdf.png" alt="A PDF Document"></div>';
                url = data.URL;
                break;
            case 'image':
                mainContent = "<img src=" + data.URL + " alt='An image'>";
                url = data.URL;
                break;
            case 'html':
                mainContent = '<div class="image-wrap blue"><img src="../Content/img/webpage.png" alt="An HTML Document"></div>';
                url = data.URL;
                break;
            case 'other':
                mainContent = '<div class="image-wrap red"><img src="../Content/img/unknown.png" alt="An HTML Document"></div>';
                url = data.URL;
                break;
            case 'doc':
                mainContent = '<div class="image-wrap orange"><img src="../Content/img/office.png" alt="An Office Document"></div>';
                url = data.URL;
                break;
        }
        var style = "";
        if (hidden) {
            style = "style='display:none;left:0;'";
        }
        var tile = "<li id='" + data.ContentItemID + "' data-id='" + data.ContentItemID + "' "+ style +">" +
            "<figure><div id='metroCheckbox' href='javascript:void(0)' data-ischecked='" + isChecked + "' data-id='" + data.ContentItemID + "'><i class='tick'></i></div>";

        tile += mainContent + "<figcaption></figcaption></figure></li>";
        return tile;
    }

    function _cleanDOM (filteredTilesIndex, animationQueueNewItems, $tiles) {
        var counter = 0;
        var filteredItemsToDeleteLength = filteredItemsToDelete.length;

        $tiles.each(function () {

            if (counter >= filteredItemsToDeleteLength) return;
            if ($(this).attr('data-id') == filteredItemsToDelete[counter]) {
                $(this).remove();
                counter++;
            }
        });


        // lastly add items from other side
        for (var idx = 0; idx < itemsToBringInAnimation.length; idx++) {
            addExisting(itemsToBringInAnimation[idx]);
        }

        var newTilesCount = animationQueueNewItems.length;
        var newTempCount = count - filteredItemsToDeleteLength;
        var space = upper - newTempCount;

        if( count > step && newTempCount + newTilesCount > upper ) {
            // there is a previous link and we need a next link space reduces by 1
            space -= 1;
        }

        if (space > 0) {
            var freeSpaces = 0;
            for (var i = 0; i < animationQueueNewItems.length; i++) {
                freeSpaces = _add(animationQueueNewItems[i].id, space, freeSpaces);
            }
        }

        if (filteredItemsToDeleteLength > 0) {
            // remove styles from items moved on screen
            for (var i = lower; i < upper; i++) {
                $($tiles[i]).removeAttr('style');
            }
        }

        // clear the list
        itemsToBringInAnimation = [];
        filteredItemsToDelete = [];

        var newCount = count - filteredTilesIndex + animationQueueNewItems.length;
        if (newCount == 0) {
            if ($('div.no-results-message').length == 0) {
                $("<div class='no-results-message' style='display:none;'>No Results</div>")
                    .appendTo('div#tiles')
                    .fadeIn(500);
            }
        } else {
            var $message = $('div.no-results-message');
            if ($message.length > 0) {
                $message.fadeOut(100, function() {
                    $message.detach();
                });
            }
        }

        updatePagination(newCount);
    }

    function _add(id, space, freeSpaces) {
        if (freeSpaces < space) {
            var $tile = $container.children('li#' + id);
            $tile.css({ 'display': 'block', 'left': '0' });
            currentlyVisibleContent.push($tile);
            var $content = $tile.children(':first');
            $content.css({ 'backgroundColor': 'rgb(0, 0, 0)', 'border': '1px solid rgb(0, 0, 0)', 'height': '0px', 'bottom': '-90px', 'width': '0px', 'left': '89px', 'margin-right': '183px', 'opacity': '0.2' });
            $content.animate({width: '180px', left: '-=89px', marginRight: '183px', opacity: '1'}, 150,
                function () {
                    $(this).animate({height: '180px', bottom: '+=90px'}, 100,
                        function () {
                            // hide background and border
                            $(this).css({ backgroundColor: "transparent", border: "1px solid transparent" });
                        });
                });

            return freeSpaces+=1;
        }
        return 0;
    };

    function shrink(ob) {
        // send order to back
        ob.element.css({ zIndex: "-1" });
        var fig = ob.element.children(':first');

        fig.children(':first').animate({height: '-=40%', width: '-=40%' }, 150,
            function () {
                $(this).animate({ opacity: '1' }, 500,
                    function () {
                        $(this).animate({ height: '+=40%', width: '+=40%' }, 150)
                    })
            });

        return fig.animate({ height: '-=90px', width: '-=90px', marginTop: '+=45px', marginLeft: '+=45px' }, 150,
            function () {
                // move hidden container
                ob.element.animate({ top: "-=" + ob.x, left: "-=" + ob.y }, 500,
                    function () {
                        fig.animate({ height: '+=90px', width: '+=90px', marginTop: '-=45px', marginLeft: '-=45px' }, 150)
                    })
            });
    }

    function move(ob) {
        return ob.element.animate(ob.animation, 800);
    }

    function removeItem(ob) {
        ob.element.remove();
    }

    function addExisting(ob) {
        ob.element.css({ 'display': 'block','left': '0'}); //Set the element left value to zero as it may have been moved left due to pagination
        currentlyVisibleContent.push(ob.element);
        var $content = ob.element.children(':first');
        $content.css({ 'backgroundColor': 'rgb(0, 0, 0)', 'border': '1px solid rgb(0, 0, 0)', 'height': '0px', 'bottom': '-90px', 'width': '0px', 'left': '89px', 'margin-right': '183px', 'opacity': '0.2' });
        $content.animate({width: '180px', left: '-=89px', marginRight: '183px', opacity: '1'}, 150,
            function () {
                $(this).animate({height: '180px', bottom: '+=90px'}, 100,
                    function () {
                        // remove background and border given to item
                        $(this).css({ backgroundColor: "transparent", border: "1px solid transparent" });
                        $(this).removeAttr('style');
                        ob.element.removeAttr('style');
                    });
            });
    }

    function gone(ob) {
        var $content = ob.element.children(':first');
        $content.css({ backgroundColor: "#000000", border: "1px solid #000000" });

        return $content.animate({ height: '0px', bottom: '-=90px' }, 100, function () {
            $(this).animate({width: '0px',left: '+=89px',marginRight: '183px',opacity: '0.2'}, 150);
        });
    }


    _collectTags = function() {
        return $('h3.tag[data-selected=0]').map(function () {
            return $(this).attr('data-tag');
        }).get().join(",");
    }

    function appendNext() {
        $next = $('<li class="next"></li>')
            .hide()
            .appendTo($nextol)
            .click(function () {
                if (canFilter) {
                    left = false;
                    right = true;
                    page++;
                    currentNumberBeingShown = 0;
                    show();
                }
            });
    }

    function appendPrevious() {
        $prev = $('<li class="prev"></li>')
            .hide()
            .appendTo($prevol)
            .click(function () {
                if (canFilter) {
                    left = true;
                    right = false;
                    page--;
                    currentNumberBeingShown = 0;
                    show();
                }
            });
    }

    function show() {
        lower = ((page - 1) * step);
        upper = lower + step;

        if (upper >= count) {
            showNext = false;
        } else {
            if (pageLoad) $next.fadeIn('fast');
            showNext = true;
        }

        if (lower >= 1) {
            if (pageLoad) $prev.fadeIn('fast');
            showPrev = true;
        } else {
            showPrev = false;
        }

        if (showNext && showPrev) {
            if (changedUpperBound) {
                lower -= (page - 2);
                upper = (lower + step) - 1;
            } else {
                upper = (lower + step) - 1;
            }
            changedUpperBound = true;
        }
        else {
            changedUpperBound = false;
            upper = (lower + step);
            var reduceOffset = page - 2;
            if (reduceOffset < 0) {
                lower = ((page - 1) * step) - 1;
            } else {
                lower -= reduceOffset;
                changedUpperBound = true;
            }
            if (lower < 0) lower = 0;
        }

        // when right is pressed
        if (!pageLoad && right) {
            if (!showNext) {
                $next.animate({ 'left': pushLeft }, speed, function () { $next.css({ 'left': '0px', 'display': 'none' }); });
            } else {
                $next.animate({ 'left': pushLeft }, speed);
            }
            if ($prev != null) $prev.animate({ 'left': pushLeft }, speed);
        }

        // when left is pressed
        if (!pageLoad && left) {
            if (!showPrev) {
                if ($prev != null) $prev.animate({ 'left': pushRight }, speed, function () { $prev.css({ 'left': '0px', 'display': 'none' }); });
            } else {
                $prev.animate({ 'left': pushRight }, speed);
            }
            if ($next != null) $next.animate({ 'left': pushRight }, speed);
        }

        var alreadyMovedStuffOff = false;

        _pushOutCurrentScreen();

        $children = $('#tiles ul li'); // reset children after a filter only

        // maybe need to reset the children here
        $children.each(function (i) {
            var child = $(this);
            if (pageLoad) child.hide();

            // things to move into the current view - i.e. between bounds
            if (i >= lower && i < upper) {
                currentlyVisibleContent.push(child);
                if (pageLoad) {
                    setTimeout(function () {
                        child.fadeIn(600)
                    }, (i - (Math.floor(i / step) * step)) * 80);
                }
                else {
                    if (right) {
                        setTimeout(function () {
                            // displace content far right then drag towards left
                            child.css({ 'left': '1200px', 'display': 'block' });
                            child.animate({ 'left': pushLeft }, speed);

                            if (showNext) {
                                $next.detach();
                                appendNext();
                                $next.css({ 'left': '1200px', 'display': 'block' });
                                $next.animate({ 'left': pushLeft }, speed, function () { $(this).removeAttr('style'); });
                                showNext = false;
                            }

                            if (showPrev) {
                                if ($prev == null) {
                                    // if no previous link exists yet
                                    appendPrevious();
                                    $prev.css({ 'left': '1200px', 'display': 'block' });
                                } else {
                                    $prev.detach();
                                    appendPrevious();
                                    $prev.css({ 'left': '1200px', 'display': 'block' });
                                }
                                $prev.animate({ 'left': pushLeft }, speed, function () { $(this).removeAttr('style'); });
                                showPrev = false;
                            }
                        }, speed);
                    }
                    else {
                        setTimeout(function () {
                            child.stop();
                            child.css({ 'display': 'block', 'left':'-1200px' });

                            child.animate({ 'left': pushRight }, speed);

                            if (showNext) {
                                $next.detach();
                                appendNext();
                                $next.css({ 'left': '-1200px', 'display': 'block' });
                                $next.animate({ 'left': pushRight }, speed, function () { $(this).removeAttr('style'); });
                                showNext = false;
                            }

                            if (showPrev) {
                                $prev.detach();
                                appendPrevious();
                                $prev.css({ 'left': '-1200px', 'display': 'block' });
                                $prev.animate({ 'left': pushRight }, speed, function () { $(this).removeAttr('style'); });
                                showPrev = false;
                            }
                        }, speed);
                    }
                }
                currentNumberBeingShown++;
            }

            // if going right
            else if (right && !pageLoad && i >= previousLowerBound - 1 && i < lower) {
                child.animate({ 'left': pushLeft }, {
                    duration: speed,
                    complete: function () {
                        $(this).css({'display': 'none', 'left':'0'});
                    }
                });
            }

            // if going left
            else if (left && !pageLoad && i >= previousLowerBound && i < previousUpperBound && !alreadyMovedStuffOff) {
                child.animate({ 'left': pushRight }, {
                    duration: speed,
                    complete: function () {
                        $(this).css('display', 'none');
                    }
                });
            }
        });

        pageLoad = false;
        previousLowerBound = lower;
        previousUpperBound = upper;

        var total = upper;
        var first = lower + 1;
        if (upper > count) total = count;
        if (count == 0) {
            first = 0;
        }

        $paginationCount.html('Viewing ' + first + " - " + total + ' of ' + count + ' items.');
    }


    function _pushOutCurrentScreen() {
        if (itemsToDelete.length > 0 && left) {
            alreadyMovedStuffOff = true;

            for (var idx = 0; idx < currentlyVisibleContent.length; idx++) {
                $(currentlyVisibleContent[idx]).animate({ 'left': pushRight }, {
                    duration: speed - 100,
                    complete: function () {
                        $(this).css({'display': 'none', 'left':'0'});
                    }
                });
            }

            // remove items from DOM
            for (var i = 0; i < itemsToDelete.length; i++) {
                for (var j = 0; j < $children.length; j++) {
                    if (itemsToDelete[i] == $($children[j]).attr('data-id')) {
                        $($children[j]).remove();
                        break;
                    }
                }
            }
            itemsToDelete = [];
        }

        // empty this to be re-filled next time
        currentlyVisibleContent = [];
    }

    function updatePagination(newCount) {
        $children = $('#tiles ul li');

        count = newCount;
        var pageTotal = 0;

        if (itemsToDelete.length > 0) {
            var go = true;
            while(go) {
                lower = ((page - 1) * step);
                upper = lower + step;

                if (count >= lower) {
                    go = false;
                } else {
                    page--;
                }
            }
            setTimeout(goBackAPage, 300);
        }

        function goBackAPage() {
            left = true;
            right = false;
            show();
        }

        // check if the upper bound is too high
        if (upper >= count) {
            if (count + itemsToDelete.length < upper) {
                if ($next != null) $next.fadeOut();
            }
            pageTotal = count;
        } else {
            pageTotal = lower + currentNumberBeingShown;
            $next.detach();
            appendNext();
            $next.css({ 'position': 'relative', 'right': '0px'});
            $next.fadeIn(100);
        }

        var viewingTotal = lower;
        if (count > 0) {
            viewingTotal += 1;
        }

        $paginationCount.html('Viewing ' + viewingTotal + " - " + pageTotal + ' of ' + count + ' items.');
        canFilter = true;
    }

    $.fn.paginate = function (method) {
        if (methods[method]) {
            return methods[method].apply( this, Array.prototype.slice.call( arguments, 1 ));
        } else if ( typeof method === 'object' || !method ) {
            return methods.init.apply(this, arguments);
        } else {
            $.error( 'Method ' +  method + ' does not exist on jQuery.paginate' );
        }
    };
})(jQuery);


;(function ($) {
    $.fn.metroCheckbox = function () {
        $(this).bind('click', function() {
            $(this).children(':first').toggle(); // toggle tick
            if ($(this).attr('data-ischecked') == 'true') {
                $(this).attr('data-ischecked', 'false');
            } else {
                $(this).attr('data-ischecked', 'true');
            }
            return false; // stop event propagation
        });
    };
})(jQuery);

(function ($) {

    $.fn.tag = function () {
        var appendToField = arguments[0].appendToField;
        var $inputField = $(arguments[0].inputField);

        // add selected tag
        $(this).bind('click', function () {
            if (canFilter) {
                if ($(this).attr('data-selected') == 1) {

                    if (appendToField) {
                        var data = $inputField.attr('value');
                        var splitData = [];
                        if (data.length != 0) {
                            splitData = data.split(",");
                        }
                        splitData.push($(this).attr('data-tag'));
                        $inputField.attr('value', splitData.join(","));
                    }

                    // disable this tag
                    $(this).addClass('tag-disabled');
                    $(this).attr('data-selected', '0');
                }
                else {

                    if (appendToField) {
                        var splitData = $inputField.attr('value').split(",");
                        var indRemove = -1;
                        for (var i = 0; i < splitData.length; i++) {
                            if (splitData[i] == $(this).attr('data-tag')) {
                                indRemove = i; break;
                            }
                        }
                        if (i >= 0) { splitData.splice(indRemove, 1); }
                        $inputField.attr('value', splitData.join(','));
                    }

                    // re-enable the tag option
                    $(this).removeClass('tag-disabled');
                    $(this).attr('data-selected', '1');
                }
            }
        });
    };
})(jQuery);
